// Copyright 2009, by the California Institute of Technology.
// ALL RIGHTS RESERVED. United States Government sponsorship acknowledged.
// Any commercial use must be negotiated with the Office of Technology Transfer
// at the California Institute of Technology.
//
// This software is subject to U. S. export control laws and regulations
// (22 C.F.R. 120-130 and 15 C.F.R. 730-774). To the extent that the software
// is subject to U.S. export control laws and regulations, the recipient has
// the responsibility to obtain export licenses or other export authority as
// may be required before exporting such information to foreign countries or
// providing access to foreign nationals.
//
// $Id$
package gov.nasa.pds.citool;

import gov.nasa.pds.citool.ingestor.CatalogDB;
import gov.nasa.pds.citool.ingestor.CatalogObject;
import gov.nasa.pds.citool.ingestor.Ingestor;
import gov.nasa.pds.citool.report.IngestReport;
import gov.nasa.pds.citool.target.Target;
import gov.nasa.pds.tools.LabelParserException;
import gov.nasa.pds.tools.constants.Constants.ProblemType;
import gov.nasa.pds.tools.label.Label;
import gov.nasa.pds.tools.label.ManualPathResolver;
import gov.nasa.pds.tools.label.parser.DefaultLabelParser;

import java.net.URI;
import java.net.URL;
import java.net.URISyntaxException;
import java.util.List;

public class CIToolIngester {
    private IngestReport report;
    private CatalogObject catObj;
    private CatalogDB catalogDb;

    public CIToolIngester(CatalogDB catalogDb, IngestReport report) {
        this.catalogDb = catalogDb;
        this.report = report;
        catObj = new CatalogObject();
        catObj.setCatalogDB(this.catalogDb, report);
    }

    /**
     * Ingest catalog file(s).
     *
     * @param target URL of the target (directory or file)
     * @throws Exception
     */
    public void ingest(Target target, boolean recurse) throws Exception {
        if (target.isDirectory()) {
            List<URL> list = target.traverse(recurse);
            this.ingest(list);
        }
        else {
        	this.ingest(target.toURL());
        }
        //catalogDb.close();
    }

    /**
     * Ingest a catalog file
     *
     * @param source URL of the catalog file.
     */
    public void ingest(URL source) {
        Label lbl = parse(source);
          if (lbl == null) {
        	System.out.println("lbl is null");
            return;
        } 
        else {
            //No problems found. Continue with ingestion.
            if (lbl.getProblems().isEmpty()) {     
            	catObj.ingest(lbl);
                report.record(lbl.getLabelURI(),lbl.getProblems());
            } 
            else {
            	LabelParserException lp = new LabelParserException(
                         lbl.getLabelURI(), null, null,
                         "ingest.source.UnParseable",
                         ProblemType.INVALID_LABEL, lbl.getLabelURI());
            	report.record(lbl.getLabelURI(), lbl.getProblems());
            }
        }
/*
        if( "PASS".equals(lbl.getStatus())) {
            log.log(new ToolsLogRecord(CIToolLevel.INFO, "Beginning catalog ingestion: "+lbl.getFilename()));
            catObj.ingest(lbl);
            //log.log(new ToolsLogRecord(CIToolLevel.DEBUG, "Completed: " + lbl.getFilename()));
        }
        else {
            if(!"PASS".equals(lbl.getStatus())) {
                log.log(new ToolsLogRecord(CIToolLevel.SEVERE,
                        "Failed to parse the file. Run tool in validation mode for more details \n\t" + source.toString() + "\n",
                        source.toString()));
            }
        }
*/
    }

    /**
     * Ingest catalog files from the given directories.
     *
     * @param sources URL of the directory of catalog files
     *
     */
    public void ingest(List<URL> sources) {
        for (int i=0; i<sources.size(); i++) {
            ingest((URL)sources.get(i));
        }
    }

    /**
     * Method to parse the PDS catalog file
     */
    public Label parse(URL url) {
    	URI uri = null;
        try {
            uri = url.toURI();
        } catch (URISyntaxException u) {
            //Ignore
        }
        ManualPathResolver resolver = new ManualPathResolver();
        resolver.setBaseURI(ManualPathResolver.getBaseURI(uri));
        //Parser must have "parser.pointers" set to false
        DefaultLabelParser parser = new DefaultLabelParser(false, true, resolver);
        Label label = null;
        try {
            label = parser.parseLabel(url);
        } catch (LabelParserException lp) {
            //Product tools library records files that have a missing
            //PDS_VERSION_ID as an error. However, we want VTool to record
            //this as a warning, so we need to instantiate a new
            //LabelParserException.
            if("parser.error.missingVersion".equals(lp.getKey())) {
                report.recordSkip(uri, new LabelParserException(
                        lp.getSourceFile(), null, null,
                        lp.getKey(), ProblemType.INVALID_LABEL_WARNING,
                        lp.getArguments()));
            }
            else {
                report.recordSkip(uri, lp);
            }
        } catch (Exception e) {
            report.recordSkip(uri, e);
        }
        return label;
    }
}
