// Copyright 2009, by the California Institute of Technology.
// ALL RIGHTS RESERVED. United States Government sponsorship acknowledged.
// Any commercial use must be negotiated with the Office of Technology Transfer
// at the California Institute of Technology.
//
// This software is subject to U. S. export control laws and regulations
// (22 C.F.R. 120-130 and 15 C.F.R. 730-774). To the extent that the software
// is subject to U.S. export control laws and regulations, the recipient has
// the responsibility to obtain export licenses or other export authority as
// may be required before exporting such information to foreign countries or
// providing access to foreign nationals.
//
// $Id: CatalogObject.java 8437 2011-02-03 21:11:35Z hyunlee $

package gov.nasa.pds.citool.ingestor;

import java.net.MalformedURLException;
import java.net.URL;
import java.io.File;
import java.util.List;
import java.util.Map;
import java.util.HashMap;
import java.util.Vector;
import java.util.Iterator;
import java.util.logging.Logger;

import gov.nasa.pds.tools.label.Label;
import gov.nasa.pds.tools.label.parser.LabelParser;
import gov.nasa.pds.tools.LabelParserException;
import gov.nasa.pds.tools.constants.Constants.ProblemType;
import gov.nasa.pds.tools.label.Statement;
import gov.nasa.pds.tools.label.ObjectStatement;
import gov.nasa.pds.tools.label.AttributeStatement;
import gov.nasa.pds.tools.util.Utility;
import gov.nasa.pds.citool.report.IngestReport;

/**
 * Class to parse a PDS catalog file and to call a CatalogDB class to ingest the
 * data into DB.
 *
 * @author hlee
 */
public class CatalogObject {

	private final static String[] CAT_OBJ_TYPES = { "MISSION",
			"INSTRUMENT_HOST", "INSTRUMENT", "DATA_SET", "REFERENCE",
			"PERSONNEL", "DATA_SET_COLLECTION", "INVENTORY", "SOFTWARE",
			"TARGET", "VOLUME", "NSSDC_DATA_SET_ID", "NSSDCDSID",
			"DIS_DATA_SET", "DATA_SET_RELEASE", "DATA_SET_HOUSEKEEPING",
			"RESOURCE", "ELEMENT_DEFINITION", "OBJECT_DEFINITION" };

	private final static String[] DS_OBJS = { "DATA_SET_HOST", "DATA_SET_INFORMATION",
			"DATA_SET_MISSION", "DATA_SET_REFERENCE_INFORMATION", "DATA_SET_TARGET"};
	private final static String[] DS_COLL_OBJS = { "DATA_SET_COLLECTION_INFO",
			"DATA_SET_COLL_ASSOC_DATA_SETS", "DATA_SET_COLLECTION_REF_INFO" };
	private final static String[] INST_OBJS = { "INSTRUMENT_INFORMATION", "INSTRUMENT_REFERENCE_INFO" };
	private final static String[] INST_HOST_OBJS = { "INSTRUMENT_HOST_INFORMATION", "INSTRUMENT_HOST_REFERENCE_INFO" };
	private final static String[] MSN_OBJS = { "MISSION_HOST", "MISSION_INFORMATION", "MISSION_REFERENCE_INFORMATION" };
	private final static String[] INV_OBJS = { "INVENTORY_DATA_SET_INFO" };
	//private final static String[] VOL_OBJS = { "DATA_PRODUCER", "CATALOG" };
	private final static String[] VOL_OBJS = { "DATA_PRODUCER" };
	private final static String[] DS_HK_OBJS = { "RESOURCE_INFORMATION" };
	private final static String[] PERS_OBJS = { "PERSONNEL_INFORMATION", "PERSONNEL_ELECTRONIC_MAIL" };
	private final static String[] SW_OBJS = { "SOFTWARE_INFORMATION", "SOFTWARE_ONLINE", "SOFTWARE_PURPOSE" };
	private final static String[] TARG_OBJS = { "TARGET_INFORMATION" };
	private final static String[] ELM_DEF_OBJS = { "ELEMENT_STANDARD_VALUE" };
	//private final static String[] OBJ_DEF_OBJS = { "OBJECT_HIER", "OBJECT_ELEMENT", "OBJECT_ALIAS"};

	private String _catObjType;
	private CatalogDB _catDb;
	private IngestReport _report;
	private Label _label;
	
	public CatalogObject() {}

	/**
	 * Sets CatalogDB object
	 *
	 * @param catDb
	 *            A CatalogDB object
	 */
	public void setCatalogDB(CatalogDB catDb, IngestReport report) {
		this._catDb = catDb;
		this._report = report;
	}

	/**
	 * Gets the catalog object from the LIST, sets to hash map, and calls
	 * CatalogDB object to ingest the data into the database.
	 *
	 * @param objList
	 *            List of the catalog object statement(s)
	 * @param pdsLabelMap
	 *            Hashmap of the PDS label keyword and value for all ATTRIBUTE
	 */
	protected void getCatalogObj(List<ObjectStatement> objList, Map<String, AttributeStatement> pdsLabelMap) {
		String objType = this._catObjType;
		Map<String, AttributeStatement> lblMap = null;
		Vector<String> subObjList = null;
		for (ObjectStatement smt: objList) {
			//log.log(new ToolsLogRecord(CIToolLevel.INFO, "\n   CATALOG OBJECT: " + objType));
			List<AttributeStatement> attrList = smt.getAttributes();
			subObjList = new Vector<String>();
			for (AttributeStatement attrSmt: attrList) {
				pdsLabelMap.put(attrSmt.getElementIdentifier(), attrSmt);
				//log.log(new ToolsLogRecord(CIToolLevel.DEBUG, "attrSmt: " + attrSmt.getElementIdentifier()));
			}
			List<ObjectStatement> objList2 = smt.getObjects();
			//log.log(new ToolsLogRecord(CIToolLevel.DEBUG, "object2 size() = " + objList2.size()));
			for (ObjectStatement smt2: objList2) {
				subObjList.add(smt2.getIdentifier().getId());

				List<AttributeStatement> objAttr = smt2.getAttributes();
				lblMap = new HashMap<String, AttributeStatement>(pdsLabelMap);
				for (AttributeStatement attrSmt: objAttr) {
					lblMap.put(attrSmt.getElementIdentifier(), attrSmt);
					//log.log(new ToolsLogRecord(CIToolLevel.DEBUG, "smt2.attrSmt: " + attrSmt.getElementIdentifier()));
				}

				List<ObjectStatement> objList3 = smt2.getObjects();
				//log.log(new ToolsLogRecord(CIToolLevel.DEBUG, "object3 size() = " + objList3.size()));
				for (ObjectStatement smt3: objList3) {
					List<AttributeStatement> objAttr2 = smt3.getAttributes();
					for (AttributeStatement attrSmt2: objAttr2) {
						lblMap.put(attrSmt2.getElementIdentifier(), attrSmt2);
						//log.log(new ToolsLogRecord(CIToolLevel.DEBUG, "smt3.attrSmt2: " + attrSmt2.getElementIdentifier()));
					}
					if (smt3.getIdentifier().getId().equalsIgnoreCase("MISSION_TARGET"))
						_catDb.ingestMsnHost(lblMap);
				}

				// if there is no object nested third times, just copy from the top level obejct
				if (lblMap==null)
					lblMap = new HashMap<String, AttributeStatement>(pdsLabelMap);

				if (smt2.getIdentifier().getId().equalsIgnoreCase("INSTRUMENT_HOST_REFERENCE_INFO"))
					_catDb.ingestInstHostRefInfoObject(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("INSTRUMENT_REFERENCE_INFO"))
					_catDb.ingestInstRefInfoObject(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_REFERENCE_INFORMATION"))
					_catDb.ingestDSRefInfoObject(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_HOST"))
					_catDb.ingestDSHost(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_COLL_ASSOC_DATA_SETS"))
					_catDb.ingestDSCollAssocDSObject(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_COLLECTION_REF_INFO"))
					_catDb.ingestDSCollDoc(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("MISSION_REFERENCE_INFORMATION"))
					_catDb.ingestMsnDoc(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_INFORMATION") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_MISSION") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_TARGET") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("DATA_SET_COLLECTION_INFO") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("PERSONNEL_INFORMATION") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("OBJECT_ELEMENT") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("OBJECT_ALIAS") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("INSTRUMENT_INFORMATION") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("SOFTWARE_INFORMATION") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("SOFTWARE_PURPOSE") ||
						smt2.getIdentifier().getId().equalsIgnoreCase("TARGET_INFORMATION"))
					pdsLabelMap = lblMap;
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("PERSONNEL_ELECTRONIC_MAIL"))
					_catDb.ingestPersElecmail(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("SOFTWARE_ONLINE"))
					_catDb.ingestSoftwareOnlineObject(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("RESOURCE_INFORMATION"))
					_catDb.ingestResourceObject(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("ELEMENT_STANDARD_VALUE"))
					_catDb.ingestStdValObject(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("OBJECT_HIER"))
					_catDb.ingestDDObjHier(lblMap);
				else if (smt2.getIdentifier().getId().equalsIgnoreCase("INVENTORY_DATA_SET_INFO"))
					_catDb.ingestInventoryObject(lblMap);
				else  // added on 10-12-10     need to test with multiple objects to make sure it works
					pdsLabelMap = lblMap;

				//if (lblMap!=null)
					//log.log(new ToolsLogRecord(CIToolLevel.DEBUG, "lblMap = " + lblMap.toString()));
			}

			if (objType.equalsIgnoreCase("PERSONNEL")) {
				_catDb.ingestPersonnelObject(pdsLabelMap);
				for (int j=0; j<PERS_OBJS.length; j++) {
					if (subObjList.indexOf(PERS_OBJS[j])==-1) {		            	
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + PERS_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, PERS_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("REFERENCE"))	{// no subObjects
				_catDb.ingestReferenceObject(pdsLabelMap);
			}
			else if (objType.equalsIgnoreCase("MISSION")) {
				_catDb.ingestMissionObject(pdsLabelMap);
				for (int j=0; j<MSN_OBJS.length; j++) {
					/*
					//if (subObjList.indexOf(MSN_OBJS[j])==-1)
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + MSN_OBJS[j] + " object is not found."));
					 */
				}
			}
			else if (objType.equalsIgnoreCase("INSTRUMENT_HOST")) {
				_catDb.ingestInstHostObject(pdsLabelMap);
				for (int j=0; j<INST_HOST_OBJS.length; j++) {
					if (subObjList.indexOf(INST_HOST_OBJS[j])==-1) {
					    //log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + INST_HOST_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, INST_HOST_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("INSTRUMENT")) {
				_catDb.ingestInstObject(pdsLabelMap);
				for (int j=0; j<INST_OBJS.length; j++) {
					if (subObjList.indexOf(INST_OBJS[j])==-1) {
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + INST_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, INST_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("DATA_SET")) {
				_catDb.ingestDataSetObject(pdsLabelMap);
				for (int j=0; j<DS_OBJS.length; j++) {
					if (subObjList.indexOf(DS_OBJS[j])==-1) {
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + DS_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, DS_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("DATA_SET_COLLECTION")) {
				_catDb.ingestDataSetColObject(pdsLabelMap);
				for (int j=0; j<DS_COLL_OBJS.length; j++) {
					if (subObjList.indexOf(DS_COLL_OBJS[j])==-1) {
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + DS_COLL_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, DS_COLL_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("SOFTWARE")) {
				_catDb.ingestSoftwareObject(pdsLabelMap);
				for (int j=0; j<SW_OBJS.length; j++) {
					if (subObjList.indexOf(SW_OBJS[j])==-1) {
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + SW_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, SW_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("TARGET")) {
				_catDb.ingestTargetObject(pdsLabelMap);
				for (int j=0; j<TARG_OBJS.length; j++) {
					if (subObjList.indexOf(TARG_OBJS[j])==-1) {
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + TARG_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, TARG_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("VOLUME")) {
				_catDb.ingestVolumeObject(pdsLabelMap);
				for (int j=0; j<VOL_OBJS.length; j++) {
					if (subObjList.indexOf(VOL_OBJS[j])==-1) {
						//log.log(new ToolsLogRecord(CIToolLevel.SEVERE, "   ERROR: " + VOL_OBJS[j] + " object is not found."));
						LabelParserException lp = new LabelParserException(_label.getLabelURI(), null,
			                    null, "ingest.error.missingObject",
			                    ProblemType.MISSING_MEMBER, VOL_OBJS[j]);
			            _label.addProblem(lp);
					}
				}
			}
			else if (objType.equalsIgnoreCase("NSSDC_DATA_SET_ID")	// no subObjects
					|| objType.equalsIgnoreCase("NSSDCDSID")) {
				_catDb.ingestNssdcdsidObject(pdsLabelMap);
			}
			else if (objType.equalsIgnoreCase("DATA_SET_RELEASE")) { // no subObjects
				_catDb.ingestDSReleaseObject(pdsLabelMap);
			}
			else if (objType.equalsIgnoreCase("ELEMENT_DEFINITION")) {
				_catDb.ingestElemDefObject(pdsLabelMap);
			}
			else if (objType.equalsIgnoreCase("OBJECT_DEFINITION")) {
				_catDb.ingestObjDefObject(pdsLabelMap);
			}
		}

		Map<String, String> standardValues = _catDb.getIngestedStdValues();
        _catDb.ingestNewStdValues(standardValues);
        //log.log(new ToolsLogRecord(CIToolLevel.WARNING, " Completed ingestion of '" + objType + "' catalog object.\n"));
	}

	/**
	 * Converts a label object to List type
	 *
	 * @param label
	 *            Label object
	 */
	protected List<ObjectStatement> object2List(Label label) {
		List<ObjectStatement> objList = null;
		for (int i = 0; i < CAT_OBJ_TYPES.length; i++) {
			if (label.getObjects(CAT_OBJ_TYPES[i]).size() > 0) {
				_catObjType = CAT_OBJ_TYPES[i];
				objList = label.getObjects(CAT_OBJ_TYPES[i]);
			}
		}
		return objList;
	}

	/**
	 * Method to convert top level attribute into the hashmap object and calls a
	 * method to convert the rest of the attributes (nested object statement)
	 */
	public void ingest(Label label) {
		this._catDb.setLabel(label);
		this._label = label;
		
		List<AttributeStatement> attrList = label.getAttributes();
		Map<String, AttributeStatement> pdsLabelMap = new HashMap<String, AttributeStatement>();
		for (Iterator i = attrList.iterator(); i.hasNext();) {
			AttributeStatement attrSmt = (AttributeStatement) i.next();
			pdsLabelMap.put(attrSmt.getElementIdentifier(), attrSmt);
			//System.out.println("attrSmt = " + attrSmt.getElementIdentifier());
		}

		List<ObjectStatement> objList = object2List(label);
		//System.out.println("before getCatalogObj....");
		getCatalogObj(objList, pdsLabelMap);

		//System.out.println("after getCatalogObj....");
		if (_catDb.isFailed()) {
			//log.log(new ToolsLogRecord(CIToolLevel.SEVERE,
					//"Failed to ingest one or more catalog object(s): " + label.getFilename() + "\n"));
			LabelParserException lp = new LabelParserException(_label.getLabelURI(), null, null, 
        			"ingest.error.failIngestion", ProblemType.EXECUTE_FAIL, _label.getLabelURI());			
            _label.addProblem(lp);
			//_report.record(_label.getLabelURI(),_label.getProblems());
			_catDb.setIsFailed(false);
			
		} 
		/*
		else {
			System.out.println("Processed: " + label.getLabelURI() + "\n");
			//log.log(new ToolsLogRecord(CIToolLevel.WARNING, "Processed: " + label.getFilename() + "\n"));
			//_report.record(_label.getLabelURI(),_label.getProblems());
		}
		*/
	}
}
